#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from unittest import TestCase
from datetime import datetime
from agspiel.api.ceo import Ceo
from agspiel.api.aktie import Aktie
from agspiel.api.anleihe import Anleihe, Kredit
from agspiel.api.zertifikat import Zertifikat
from agspiel.api.order import Order
from agspiel.api.api import Api
from bs4 import BeautifulSoup

class TestAg(TestCase):
    def setUp(self):
        ceo = {"name":"KingKevin23", "registrierung_datum":"2020-02-04 16:24:00", "gesperrt":"false", "ist_userprojekt_account":"false"}
        aktien = [{"wkn":"104531", "stueckzahl":"146"}, {"wkn":"105516", "stueckzahl":"23"}]
        anleihen = [{"betrag":"10000000", "zins":"0.32", "auszahlung_datum":"2020-08-14 15:01:33", "laufzeit":"5"}]
        kredite = [{"betrag":"10000000", "zins":"0.32", "rueckzahlung_datum":"2020-08-14 15:01:33", "laufzeit":"5"}]
        zertifikate = [{"betrag":"234.43", "typ":"call", "hebel":"1.092", "punkte":"23443", "ablauf_datum":"2020-08-10 17:48:01"}]
        orders = [{"typ":"sell", "limit":"410", "stueckzahl":"303", "orderregel":"false", "systembank_order":"false", "datum":"2020-08-10 01:21:35"}]
        api_data = {"wkn":"175353", "name":"King Kompany", "gruendung":"2020-02-04 16:25:05", "aktienanzahl":"2000000",
                    "in_liquidation":"false", "kurs":"338.68", "brief":"338.68", "brief_stueckzahl":"117391", "geld":"0",
                    "geld_stueckzahl":"0", "depotwert":"162516884.99", "bargeld":"5378562.51", "highscore_platz":"528",
                    "highscore_platz_groesse":"337", "highscore_platz_wachstum":"502", "highscore_platz_newcomer":"0",
                    "agsx_punkte":"1026", "in_agsx":"false", "handelsaktivitaet":"42", "ceo":ceo, "aktien":aktien,
                    "anleihen":anleihen, "kredite":kredite, "zertifikate":zertifikate, "orders":orders}
        f = open("testpage.txt", "rb")
        web_data = BeautifulSoup(f.read(), "html.parser")
        f.close()
        self.ag = Api._create_ag(api_data, web_data)

    def test_wkn(self):
        self.assertEqual(self.ag.wkn, 175353)
        self.assertIsInstance(self.ag.wkn, int)

    def test_name(self):
        self.assertEqual(self.ag.name, "King Kompany")
        self.assertIsInstance(self.ag.name, str)

    def test_gruendung(self):
        self.assertEqual(self.ag.gruendung, datetime(year=2020, month=2, day=4, hour=16, minute=25, second=5))
        self.assertIsInstance(self.ag.gruendung, datetime)

    def test_aktienanzahl(self):
        self.assertEqual(self.ag.aktienanzahl, 2000000)
        self.assertIsInstance(self.ag.aktienanzahl, int)

    def test_in_liquidation(self):
        self.assertEqual(self.ag.in_liquidation, False)
        self.assertIsInstance(self.ag.in_liquidation, bool)

    def test_kurs(self):
        self.assertEqual(self.ag.kurs, 338.68)
        self.assertIsInstance(self.ag.kurs, float)

    def test_brief(self):
        self.assertEqual(self.ag.brief, 338.68)
        self.assertIsInstance(self.ag.brief, float)

    def test_geld(self):
        self.assertEqual(self.ag.geld, 0)
        self.assertIsInstance(self.ag.geld, float)

    def test_brief_stueckzahl(self):
        self.assertEqual(self.ag.brief_stueckzahl, 117391)
        self.assertIsInstance(self.ag.brief_stueckzahl, int)

    def test_geld_stueckzahl(self):
        self.assertEqual(self.ag.geld_stueckzahl, 0)
        self.assertIsInstance(self.ag.geld_stueckzahl, int)

    def test_sw_aktie(self):
        self.assertEqual(self.ag.sw_aktie, 52.30)
        self.assertIsInstance(self.ag.sw_aktie, float)

    def test_bbw_aktie(self):
        self.assertEqual(self.ag.bbw_aktie, 59.01)
        self.assertIsInstance(self.ag.bbw_aktie, float)

    def test_fp_aktie(self):
        self.assertEqual(self.ag.fp_aktie, 56.14)
        self.assertIsInstance(self.ag.fp_aktie, float)

    def test_bw_aktie(self):
        self.assertEqual(self.ag.bw_aktie, 83.95)
        self.assertIsInstance(self.ag.bw_aktie, float)

    def test_kgv(self):
        self.assertEqual(self.ag.kgv, 115.59)
        self.assertIsInstance(self.ag.kgv, float)

    def test_spread(self):
        self.assertEqual(self.ag.spread, 0)
        try:
            self.assertIsInstance(self.ag.spread, float)
        except AssertionError:
            self.assertEqual(self.ag.spread, 0)
            self.assertIsInstance(self.ag.spread, int)

    def test_alter(self):
        self.assertEqual(self.ag.alter, (datetime.now() - self.ag.gruendung).days)
        self.assertIsInstance(self.ag.alter, int)

    def test_tagesvolumen(self):
        self.assertEqual(self.ag.tagesvolumen, 0)
        self.assertIsInstance(self.ag.tagesvolumen, float)

    def test_boersenwert(self):
        self.assertEqual(self.ag.boersenwert, 677360000)
        self.assertIsInstance(self.ag.boersenwert, float)

    def test_buchwert(self):
        self.assertEqual(self.ag.buchwert, 167895447.5)
        self.assertIsInstance(self.ag.buchwert, float)

    def test_depotwert(self):
        self.assertEqual(self.ag.depotwert, 162516884.99)
        self.assertIsInstance(self.ag.depotwert, float)

    def test_bargeld(self):
        self.assertEqual(self.ag.bargeld, 5378562.51)
        self.assertIsInstance(self.ag.bargeld, float)

    def test_highscore(self):
        self.assertEqual(self.ag.highscore, 528)
        self.assertIsInstance(self.ag.highscore, int)

    def test_highscore_groesse(self):
        self.assertEqual(self.ag.highscore_groesse, 337)
        self.assertIsInstance(self.ag.highscore_groesse, int)

    def test_highscore_wachstum(self):
        self.assertEqual(self.ag.highscore_wachstum, 502)
        self.assertIsInstance(self.ag.highscore_wachstum, int)

    def test_highscore_newcomer(self):
        self.assertEqual(self.ag.highscore_newcomer, 0)
        self.assertIsInstance(self.ag.highscore_newcomer, int)

    def test_agsx_punkte(self):
        self.assertEqual(self.ag.agsx_punkte, 1026)
        self.assertIsInstance(self.ag.agsx_punkte, int)

    def test_in_agsx(self):
        self.assertEqual(self.ag.in_agsx, False)
        self.assertIsInstance(self.ag.in_agsx, bool)

    def test_handelsaktivitaet(self):
        self.assertEqual(self.ag.handelsaktivitaet, 42)
        self.assertIsInstance(self.ag.handelsaktivitaet, int)

    def test_ceo(self):
        self.assertIsInstance(self.ag.ceo, Ceo)
        self.assertEqual(self.ag.ceo.name, "KingKevin23")
        self.assertIsInstance(self.ag.ceo.name, str)
        self.assertEqual(self.ag.ceo.index, "Freibeuter")
        self.assertIsInstance(self.ag.ceo.index, str)
        self.assertEqual(self.ag.ceo.registrierung_datum, datetime(year=2020, month=2, day=4, hour=16, minute=24))
        self.assertIsInstance(self.ag.ceo.registrierung_datum, datetime)
        self.assertEqual(self.ag.ceo.gesperrt, False)
        self.assertIsInstance(self.ag.ceo.gesperrt, bool)
        self.assertEqual(self.ag.ceo.userprojekt, False)
        self.assertIsInstance(self.ag.ceo.userprojekt, bool)

    def test_aktien(self):
        for i in self.ag.aktien:
            self.assertIsInstance(i, Aktie)
        self.assertEqual(self.ag.aktien[0].wkn, 104531)
        self.assertIsInstance(self.ag.aktien[0].wkn, int)
        self.assertEqual(self.ag.aktien[0].stueckzahl, 146)
        self.assertIsInstance(self.ag.aktien[0].stueckzahl, int)
        self.assertEqual(self.ag.aktien[1].wkn, 105516)
        self.assertIsInstance(self.ag.aktien[1].wkn, int)
        self.assertEqual(self.ag.aktien[1].stueckzahl, 23)
        self.assertIsInstance(self.ag.aktien[1].stueckzahl, int)

    def test_anleihen(self):
        for i in self.ag.anleihen:
            self.assertIsInstance(i, Anleihe)
        self.assertEqual(self.ag.anleihen[0].betrag, 10000000)
        self.assertIsInstance(self.ag.anleihen[0].betrag, int)
        self.assertEqual(self.ag.anleihen[0].zins, 0.32)
        self.assertIsInstance(self.ag.anleihen[0].zins, float)
        self.assertEqual(self.ag.anleihen[0].auszahlung_datum, datetime(year=2020, month=8, day=14, hour=15, minute=1, second=33))
        self.assertIsInstance(self.ag.anleihen[0].auszahlung_datum, datetime)
        self.assertEqual(self.ag.anleihen[0].laufzeit, 5)
        self.assertIsInstance(self.ag.anleihen[0].laufzeit, int)

    def test_kredite(self):
        for i in self.ag.kredite:
            self.assertIsInstance(i, Kredit)
        self.assertEqual(self.ag.kredite[0].betrag, 10000000)
        self.assertIsInstance(self.ag.kredite[0].betrag, int)
        self.assertEqual(self.ag.kredite[0].zins, 0.32)
        self.assertIsInstance(self.ag.kredite[0].zins, float)
        self.assertEqual(self.ag.kredite[0].rueckzahlung_datum, datetime(year=2020, month=8, day=14, hour=15, minute=1, second=33))
        self.assertIsInstance(self.ag.kredite[0].rueckzahlung_datum, datetime)
        self.assertEqual(self.ag.kredite[0].laufzeit, 5)
        self.assertIsInstance(self.ag.kredite[0].laufzeit, int)

    def test_zertifikate(self):
        for i in self.ag.zertifikate:
            self.assertIsInstance(i, Zertifikat)
        self.assertEqual(self.ag.zertifikate[0].betrag, 234.43)
        self.assertIsInstance(self.ag.zertifikate[0].betrag, float)
        self.assertEqual(self.ag.zertifikate[0].typ, "call")
        self.assertIsInstance(self.ag.zertifikate[0].typ, str)
        self.assertEqual(self.ag.zertifikate[0].hebel, 1.092)
        self.assertIsInstance(self.ag.zertifikate[0].hebel, float)
        self.assertEqual(self.ag.zertifikate[0].punkte, 23443)
        self.assertIsInstance(self.ag.zertifikate[0].punkte, int)
        self.assertEqual(self.ag.zertifikate[0].ablaufdatum, datetime(year=2020, month=8, day=10, hour=17, minute=48, second=1))
        self.assertIsInstance(self.ag.zertifikate[0].ablaufdatum, datetime)

    def test_orders(self):
        for i in self.ag.orders:
            self.assertIsInstance(i, Order)
        self.assertEqual(self.ag.orders[0].typ, "sell")
        self.assertIsInstance(self.ag.orders[0].typ, str)
        self.assertEqual(self.ag.orders[0].limit, 410)
        self.assertIsInstance(self.ag.orders[0].limit, float)
        self.assertEqual(self.ag.orders[0].stueckzahl, 303)
        self.assertIsInstance(self.ag.orders[0].stueckzahl, int)
        self.assertEqual(self.ag.orders[0].orderregel, False)
        self.assertIsInstance(self.ag.orders[0].orderregel, bool)
        self.assertEqual(self.ag.orders[0].systembank, False)
        self.assertIsInstance(self.ag.orders[0].systembank, bool)
        self.assertEqual(self.ag.orders[0].datum, datetime(year=2020, month=8, day=10, hour=1, minute=21, second=35))
        self.assertIsInstance(self.ag.orders[0].datum, datetime)

    def test_dividende(self):
        self.assertEqual(self.ag.dividende, 0)
        self.assertIsInstance(self.ag.dividende, float)

    def test_max_zertis(self):
        self.assertEqual(self.ag.max_zertis, 5)
        self.assertIsInstance(self.ag.max_zertis, int)

    def test_tages_hoch(self):
        self.assertEqual(self.ag.tages_hoch, 423.35)
        self.assertIsInstance(self.ag.tages_hoch, float)

    def test_tages_tief(self):
        self.assertEqual(self.ag.tages_tief, 338.68)
        self.assertIsInstance(self.ag.tages_tief, float)

    def test_kurs_14d(self):
        self.assertEqual(self.ag.kurs_14d, 38.24)
        self.assertIsInstance(self.ag.kurs_14d, float)

    def test_kurs_30d(self):
        self.assertEqual(self.ag.kurs_30d, -0.39)
        self.assertIsInstance(self.ag.kurs_30d, float)

    def test_kurs_60d(self):
        self.assertEqual(self.ag.kurs_60d, 47.26)
        self.assertIsInstance(self.ag.kurs_60d, float)

    def test_kurs_90d(self):
        self.assertEqual(self.ag.kurs_90d, 35.47)
        self.assertIsInstance(self.ag.kurs_90d, float)

    def test_bw_14d(self):
        self.assertEqual(self.ag.bw_14d, -12.32)
        self.assertIsInstance(self.ag.bw_14d, float)

    def test_bw_30d(self):
        self.assertEqual(self.ag.bw_30d, 4.74)
        self.assertIsInstance(self.ag.bw_30d, float)

    def test_bw_60d(self):
        self.assertEqual(self.ag.bw_60d, 28.03)
        self.assertIsInstance(self.ag.bw_60d, float)

    def test_bw_90d(self):
        self.assertEqual(self.ag.bw_90d, 68.96)
        self.assertIsInstance(self.ag.bw_90d, float)

    def test_fp_14d(self):
        self.assertEqual(self.ag.fp_14d, 3.89)
        self.assertIsInstance(self.ag.fp_14d, float)

    def test_fp_30d(self):
        self.assertEqual(self.ag.fp_30d, 5.51)
        self.assertIsInstance(self.ag.fp_30d, float)

    def test_fp_60d(self):
        self.assertEqual(self.ag.fp_60d, 11.74)
        self.assertIsInstance(self.ag.fp_60d, float)

    def test_fp_90d(self):
        self.assertEqual(self.ag.fp_90d, 16.59)
        self.assertIsInstance(self.ag.fp_90d, float)


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from datetime import datetime
from .ceo import Ceo

class Ag:
    def __init__(self, wkn:int=None, name:str=None, gruendung:datetime=None, aktienanzahl:int=None,
                 in_liquidation:bool=None, kurs:float=None, brief:float=None, geld:float=None, brief_stueckzahl:int=None,
                 geld_stueckzahl:int=None, sw_aktie:float=None, bbw_aktie:float=None, fp_aktie:float=None,
                 kgv:float=None, tagesvolumen:float=None, depotwert:float=None, bargeld:float=None, highscore:int=None,
                 highscore_groesse:int=None, highscore_wachstum:int=None, highscore_newcomer:int=None,
                 agsx_punkte:int=None, in_agsx:bool=None, handelsaktivitaet:int=None, ceo:Ceo=None, aktien:list=None,
                 anleihen:list=None, kredite:list=None, zertifikate:list=None, orders:list=None, dividende:float=None,
                 max_zertis:int=None, tages_hoch:float=None, tages_tief:float=None, kurs_14d:float=None,
                 kurs_30d:float=None, kurs_60d:float=None, kurs_90d:float=None, bw_14d:float=None, bw_30d:float=None,
                 bw_60d:float=None, bw_90d:float=None, fp_14d:float=None, fp_30d:float=None, fp_60d:float=None,
                 fp_90d:float=None):
        self.wkn:int = wkn
        self.name:str = name
        self.gruendung:datetime = gruendung
        self.aktienanzahl:int = aktienanzahl
        self.in_liquidation:bool = in_liquidation
        self.kurs:float = kurs
        self.brief:float = brief
        self.geld:float = geld
        self.brief_stueckzahl:int = brief_stueckzahl
        self.geld_stueckzahl:int = geld_stueckzahl
        self.sw_aktie:float = sw_aktie
        self.bbw_aktie:float = bbw_aktie
        self.fp_aktie:float = fp_aktie
        self.kgv:float = kgv
        self.tagesvolumen:float = tagesvolumen
        self.depotwert:float = depotwert
        self.bargeld:float = bargeld
        self.highscore:int = highscore
        self.highscore_groesse:int = highscore_groesse
        self.highscore_wachstum:int = highscore_wachstum
        self.highscore_newcomer:int = highscore_newcomer
        self.agsx_punkte:int = agsx_punkte
        self.in_agsx:bool = in_agsx
        self.handelsaktivitaet:int = handelsaktivitaet
        self.ceo:Ceo = ceo
        self.aktien:list = aktien
        self.anleihen:list = anleihen
        self.kredite:list = kredite
        self.zertifikate:list = zertifikate
        self.orders:list = orders
        self.dividende:float = dividende
        self.max_zertis:int = max_zertis
        self.tages_hoch:float = tages_hoch
        self.tages_tief:float = tages_tief
        self.kurs_14d:float = kurs_14d
        self.kurs_30d:float = kurs_30d
        self.kurs_60d:float = kurs_60d
        self.kurs_90d:float = kurs_90d
        self.bw_14d:float = bw_14d
        self.bw_30d:float = bw_30d
        self.bw_60d:float = bw_60d
        self.bw_90d:float = bw_90d
        self.fp_14d:float = fp_14d
        self.fp_30d:float = fp_30d
        self.fp_60d:float = fp_60d
        self.fp_90d:float = fp_90d

        # Hier werden alle Kennzahlen initialisiert, die aus anderen berechnet werden können.
        # Genaue Berechnung erfolgt dann im Getter
        self.alter:int
        self.boersenwert:float
        self.buchwert:float
        self.spread:float
        self.bw_aktie:float

    @property
    def wkn(self) -> int:
        return self._wkn

    @wkn.setter
    def wkn(self, value:int):
        self._wkn = value

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value:str):
        self._name = value

    @property
    def gruendung(self) -> datetime:
        return self._gruendung

    @gruendung.setter
    def gruendung(self, value:datetime):
        self._gruendung = value

    @property
    def aktienanzahl(self) -> int:
        return self._aktienanzahl

    @aktienanzahl.setter
    def aktienanzahl(self, value:int):
        self._aktienanzahl = value

    @property
    def in_liquidation(self) -> bool:
        return self._in_liquidation

    @in_liquidation.setter
    def in_liquidation(self, value:bool):
        self._in_liquidation = value

    @property
    def kurs(self) -> float:
        return self._kurs

    @kurs.setter
    def kurs(self, value:float):
        self._kurs = value

    @property
    def brief(self) -> float:
        return self._brief

    @brief.setter
    def brief(self, value:float):
        self._brief = value

    @property
    def geld(self) -> float:
        return self._geld

    @geld.setter
    def geld(self, value:float):
        self._geld = value

    @property
    def brief_stueckzahl(self) -> int:
        return self._brief_stueckzahl

    @brief_stueckzahl.setter
    def brief_stueckzahl(self, value:int):
        self._brief_stueckzahl = value

    @property
    def geld_stueckzahl(self) -> int:
        return self._geld_stueckzahl

    @geld_stueckzahl.setter
    def geld_stueckzahl(self, value:int):
        self._geld_stueckzahl = value

    @property
    def sw_aktie(self) -> float:
        return self._sw_aktie

    @sw_aktie.setter
    def sw_aktie(self, value:float):
        self._sw_aktie = value

    @property
    def bbw_aktie(self) -> float:
        return self._bbw_aktie

    @bbw_aktie.setter
    def bbw_aktie(self, value:float):
        self._bbw_aktie = value

    @property
    def fp_aktie(self) -> float:
        return self._fp_aktie

    @fp_aktie.setter
    def fp_aktie(self, value:float):
        self._fp_aktie = value

    @property
    def bw_aktie(self) -> float:
        return round(self.buchwert / self.aktienanzahl, 2)

    @bw_aktie.setter
    def bw_aktie(self, value:float):
        self._bw_aktie = value

    @property
    def kgv(self) -> float:
        return self._kgv

    @kgv.setter
    def kgv(self, value:float):
        self._kgv = value

    @property
    def spread(self) -> float:
        if self.geld == 0 or self.brief == 0:
            return 0
        else:
            return 1 - (self.geld / self.brief)

    @spread.setter
    def spread(self, value:float):
        self._spread = value

    @property
    def alter(self) -> int:
        return (datetime.now() - self.gruendung).days

    @alter.setter
    def alter(self, value:int):
        self._alter = value

    @property
    def tagesvolumen(self) -> float:
        return self._tagesvolumen

    @tagesvolumen.setter
    def tagesvolumen(self, value:float):
        self._tagesvolumen = value

    @property
    def boersenwert(self) -> float:
        return self.kurs * self.aktienanzahl

    @boersenwert.setter
    def boersenwert(self, value:float):
        self._boersenwert = value

    @property
    def buchwert(self) -> float:
        return self.depotwert + self.bargeld

    @buchwert.setter
    def buchwert(self, value:float):
        self._buchwert = value

    @property
    def depotwert(self) -> float:
        return self._depotwert

    @depotwert.setter
    def depotwert(self, value:float):
        self._depotwert = value

    @property
    def bargeld(self) -> float:
        return self._bargeld

    @bargeld.setter
    def bargeld(self, value:float):
        self._bargeld = value

    @property
    def highscore(self) -> int:
        return self._highscore

    @highscore.setter
    def highscore(self, value:int):
        self._highscore = value

    @property
    def highscore_groesse(self) -> int:
        return self._highscore_groesse

    @highscore_groesse.setter
    def highscore_groesse(self, value:int):
        self._highscore_groesse = value

    @property
    def highscore_wachstum(self) -> int:
        return self._highscore_wachstum

    @highscore_wachstum.setter
    def highscore_wachstum(self, value:int):
        self._highscore_wachstum = value

    @property
    def highscore_newcomer(self) -> int:
        return self._highscore_newcomer

    @highscore_newcomer.setter
    def highscore_newcomer(self, value:int):
        self._highscore_newcomer = value

    @property
    def agsx_punkte(self) -> int:
        return self._agsx_punkte

    @agsx_punkte.setter
    def agsx_punkte(self, value:int):
        self._agsx_punkte = value

    @property
    def in_agsx(self) -> bool:
        return self._in_agsx

    @in_agsx.setter
    def in_agsx(self, value:bool):
        self._in_agsx = value

    @property
    def handelsaktivitaet(self) -> int:
        return self._handelsaktivitaet

    @handelsaktivitaet.setter
    def handelsaktivitaet(self, value:int):
        self._handelsaktivitaet = value

    @property
    def ceo(self) -> Ceo:
        return self._ceo

    @ceo.setter
    def ceo(self, value:Ceo):
        self._ceo = value

    @property
    def aktien(self) -> list:
        return self._aktien

    @aktien.setter
    def aktien(self, value:list):
        self._aktien = value

    @property
    def anleihen(self) -> list:
        return self._anleihen

    @anleihen.setter
    def anleihen(self, value:list):
        self._anleihen = value

    @property
    def kredite(self) -> list:
        return self._kredite

    @kredite.setter
    def kredite(self, value:list):
        self._kredite = value

    @property
    def zertifikate(self) -> list:
        return self._zertifikate

    @zertifikate.setter
    def zertifikate(self, value:list):
        self._zertifikate = value

    @property
    def orders(self) -> list:
        return self._orders

    @orders.setter
    def orders(self, value:list):
        self._orders = value

    @property
    def dividende(self) -> float:
        return self._dividende

    @dividende.setter
    def dividende(self, value:float):
        self._dividende = value

    @property
    def max_zertis(self) -> int:
        return self._max_zertis

    @max_zertis.setter
    def max_zertis(self, value:int):
        self._max_zertis = value

    @property
    def tages_hoch(self) -> float:
        return self._tages_hoch

    @tages_hoch.setter
    def tages_hoch(self, value:float):
        self._tages_hoch = value

    @property
    def tages_tief(self) -> float:
        return self._tages_tief

    @tages_tief.setter
    def tages_tief(self, value:float):
        self._tages_tief = value

    @property
    def kurs_14d(self) -> float:
        return self._kurs_14d

    @kurs_14d.setter
    def kurs_14d(self, value:float):
        self._kurs_14d = value

    @property
    def kurs_30d(self) -> float:
        return self._kurs_30d

    @kurs_30d.setter
    def kurs_30d(self, value:float):
        self._kurs_30d = value

    @property
    def kurs_60d(self) -> float:
        return self._kurs_60d

    @kurs_60d.setter
    def kurs_60d(self, value:float):
        self._kurs_60d = value

    @property
    def kurs_90d(self) -> float:
        return self._kurs_90d

    @kurs_90d.setter
    def kurs_90d(self, value:float):
        self._kurs_90d = value

    @property
    def bw_14d(self) -> float:
        return self._bw_14d

    @bw_14d.setter
    def bw_14d(self, value:float):
        self._bw_14d = value

    @property
    def bw_30d(self) -> float:
        return self._bw_30d

    @bw_30d.setter
    def bw_30d(self, value:float):
        self._bw_30d = value

    @property
    def bw_60d(self) -> float:
        return self._bw_60d

    @bw_60d.setter
    def bw_60d(self, value:float):
        self._bw_60d = value

    @property
    def bw_90d(self) -> float:
        return self._bw_90d

    @bw_90d.setter
    def bw_90d(self, value:float):
        self._bw_90d = value

    @property
    def fp_14d(self) -> float:
        return self._fp_14d

    @fp_14d.setter
    def fp_14d(self, value:float):
        self._fp_14d = value

    @property
    def fp_30d(self) -> float:
        return self._fp_30d

    @fp_30d.setter
    def fp_30d(self, value:float):
        self._fp_30d = value

    @property
    def fp_60d(self) -> float:
        return self._fp_60d

    @fp_60d.setter
    def fp_60d(self, value:float):
        self._fp_60d = value

    @property
    def fp_90d(self) -> float:
        return self._fp_90d

    @fp_90d.setter
    def fp_90d(self, value:float):
        self._fp_90d = value


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

import requests, json, re
from .ag import Ag
from .markt import Markt
from datetime import datetime
from bs4 import BeautifulSoup
from .ceo import Ceo
from .aktie import Aktie
from .anleihe import Anleihe, Kredit
from .zertifikat import Zertifikat
from .order import Order

class Api:
    _api_url = "https://www.ag-spiel.de/api/get/data.php?version=5"

    def __init__(self, phpsessid:str, newest_data:bool=False):
        self._phpsessid = phpsessid
        self._newest_data = newest_data
        self._data = None

    def get_ag(self, wkn:int) -> Ag:
        """
        Diese Methode gibt ein Objekt der Klasse Ag mit der übergebenen WKN aus.

        :param wkn: Die WKN der gewünschten Ag
        :return: Ein Objekt der Klasse Ag
        """
        data = self._get_data().get("ags").get(str(wkn))
        web = requests.get("https://www.ag-spiel.de/index.php?section=profil&aktie={}".format(str(wkn)),
                                                       cookies={"PHPSESSID":self._phpsessid}).content
        return Api._create_ag(api_data=data, web_data=BeautifulSoup(web, "html.parser"))

    def get_markt(self) -> Markt:
        data = self._get_data().get("allgemein")
        web = requests.get("https://www.ag-spiel.de/index.php?section=login").content
        return Api._create_markt(api_data=data, web_data=BeautifulSoup(web, "html.parser"))

    @property
    def api_version(self) -> int:
        return int(self._get_data().get("api_version"))

    @property
    def daten_datum(self) -> datetime:
        return datetime.strptime(self._get_data().get("daten_datum"), "%Y-%m-%d %H:%M:%S")

    def _get_data(self) -> dict:
        """
        Diese Methode holt sich die aktuellen Daten aus der AGS-API.

        :return: Ein Dictionary mit den API-Daten
        """
        if self._newest_data:
            return json.loads(requests.get(Api._api_url).content)
        else:
            if self._data is None:
                self._data = json.loads(requests.get(Api._api_url).content)

            return self._data

    @staticmethod
    def _create_markt(api_data:dict, web_data:BeautifulSoup) -> Markt:
        markt = Markt()
        markt.ags = int(api_data.get("ags"))
        markt.orders_24 = int(api_data.get("24_stunden_orders"))
        markt.volumen_24 = float(api_data.get("24_stunden_ordervolumen"))

        table_data = {}
        rows = web_data.find('table', attrs={'class': 'menu2'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            try:
                table_data[cols[0].text] = cols[1].text
            except IndexError:
                pass

        markt.agsx_punkte = int(table_data.get("Punktestand").replace(".", ""))
        markt.agsx_aenderung = int(table_data.get("Änderung"))
        markt.put_hebel = float(re.compile("(\d\.?\d*)\s/\s\d\.?\d*").findall(table_data.get("Put / Call"))[0])
        markt.call_hebel = float(re.compile("\d\.?\d*\s/\s(\d\.?\d*)").findall(table_data.get("Put / Call"))[0])
        markt.anleihenzins = float(re.compile("(\d\.?\d*)%").findall(table_data.get("Anleihezins"))[0])

        return markt

    @staticmethod
    def _create_ag(api_data:dict, web_data:BeautifulSoup) -> Ag:
        """
        Diese Methode erstellt ein Objekt der Klasse AG und befüllt dieses mit den übergebenen Werten.

        :param api_data: Ein Dictionary, welches die Daten aus der API enthält
        :param web_data: Ein BeatifulSoup Objekt, welches die Website der AG enthält
        :return: Ein Objekt der Klasse AG, welches die gewünschten Werte enthält
        """
        ag = Ag()
        ag.wkn = int(api_data.get("wkn"))
        ag.name = api_data.get("name")
        ag.gruendung = datetime.strptime(api_data.get("gruendung"), "%Y-%m-%d %H:%M:%S")
        ag.aktienanzahl = int(api_data.get("aktienanzahl"))
        ag.in_liquidation = api_data.get("in_liquidation") == "true"
        ag.kurs = float(api_data.get("kurs"))
        ag.brief = float(api_data.get("brief"))
        ag.geld = float(api_data.get("geld"))
        ag.brief_stueckzahl = int(api_data.get("brief_stueckzahl"))
        ag.geld_stueckzahl = int(api_data.get("geld_stueckzahl"))
        ag.sw_aktie = float(re.compile("-?\d*[.]?\d*[,]\d{2}").findall(web_data.find("div", attrs={"id":"sw"}).text)[0]
                            .replace(".", "").replace(",", "."))
        ag.bbw_aktie = float(re.compile("-?\d*[.]?\d*[,]\d{2}").findall(web_data.find("div", attrs={"id": "bbw"}).text)[0]
                             .replace(".", "").replace(",", "."))
        ag.fp_aktie = float(re.compile("-?\d*[.]?\d*[,]\d{2}").findall(web_data.find("div", attrs={"id": "fp"}).text)[0]
                            .replace(".", "").replace(",", "."))
        ag.kgv = float(re.compile("\d*[,]\d*").findall(web_data.find("div", attrs={"id":"kgv"}).text)[0].
                       replace(",", "."))
        ag.tagesvolumen = Api._convert_number(web_data.find("div", attrs={"id":"tagesvolumen"}).text)
        ag.depotwert = float(api_data.get("depotwert"))
        ag.bargeld = float(api_data.get("bargeld"))
        ag.highscore = int(api_data.get("highscore_platz"))
        ag.highscore_groesse = int(api_data.get("highscore_platz_groesse"))
        ag.highscore_wachstum = int(api_data.get("highscore_platz_wachstum"))
        ag.highscore_newcomer = int(api_data.get("highscore_platz_newcomer"))
        ag.agsx_punkte = int(api_data.get("agsx_punkte"))
        ag.in_agsx = api_data.get("in_agsx") == "true"
        ag.handelsaktivitaet = int(api_data.get("handelsaktivitaet"))
        ag.ceo = Api._create_ceo(ceo_data=api_data.get("ceo"), web_data=web_data)

        ag.aktien = []
        for i in api_data.get("aktien"):
            temp = Aktie(wkn=int(i.get("wkn")), stueckzahl=int(i.get("stueckzahl")))
            ag.aktien.append(temp)

        ag.anleihen = []
        for i in api_data.get("anleihen"):
            temp = Anleihe(betrag=int(i.get("betrag")), zins=float(i.get("zins")),
                           auszahlung_datum=datetime.strptime(i.get("auszahlung_datum"), "%Y-%m-%d %H:%M:%S"),
                           laufzeit=int(i.get("laufzeit")))
            ag.anleihen.append(temp)

        ag.kredite = []
        for i in api_data.get("kredite"):
            temp = Kredit(betrag=int(i.get("betrag")), zins=float(i.get("zins")),
                          rueckzahlung_datum=datetime.strptime(i.get("rueckzahlung_datum"), "%Y-%m-%d %H:%M:%S"),
                          laufzeit=int(i.get("laufzeit")))
            ag.kredite.append(temp)

        ag.zertifikate = []
        for i in api_data.get("zertifikate"):
            try:
                hebel = float(i.get("hebel"))
            except TypeError:
                hebel = 0
            typ = i.get("typ")
            temp = Zertifikat(betrag=float(i.get("betrag")), typ=typ, hebel=hebel, punkte=int(i.get("punkte")),
                              ablaufdatum=datetime.strptime(i.get("ablauf_datum"), "%Y-%m-%d %H:%M:%S"))
            ag.zertifikate.append(temp)

        ag.orders = []
        for i in api_data.get("orders"):
            temp = Order(typ=i.get("typ"), limit=float(i.get("limit")), stueckzahl=int(i.get("stueckzahl")),
                         orderregel=i.get("orderregel") == "true", systembank=i.get("systembank_order") == "true",
                         datum=datetime.strptime(i.get("datum"), "%Y-%m-%d %H:%M:%S"))
            ag.orders.append(temp)

        table_data = []
        rows = web_data.find('table', attrs={'class': 'normalborder'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            for col in cols:
                inhalt = col.find_all("span")
                try:
                    table_data.append(float(inhalt[0].text.replace(",", ".")))
                except IndexError:
                    pass

        offset = int(len(table_data) / 3) - 1 # Die Länge sollte immer glatt durch 3 teilbar sein

        # Fall das len = 0 ist muss nicht beachtet werden, da Werte standardmäßig None
        if len(table_data) >= 3:
            ag.kurs_14d = table_data[0]
            ag.bw_14d = table_data[1 + offset]
            ag.fp_14d = table_data[2 + offset * 2]
        if len(table_data) >= 6:
            ag.kurs_30d = table_data[1]
            ag.bw_30d = table_data[2 + offset]
            ag.fp_30d = table_data[3 + offset * 2]
        if len(table_data) >= 9:
            ag.kurs_60d = table_data[2]
            ag.bw_60d = table_data[3 + offset]
            ag.fp_60d = table_data[4 + offset * 2]
        if len(table_data) >= 12:
            ag.kurs_90d = table_data[3]
            ag.bw_90d = table_data[4 + offset]
            ag.fp_90d = table_data[5 + offset * 2]


        table_data = {}
        rows = web_data.find('table', attrs={'class': 'padding5'}).find_all('tr')
        for row in rows:
            cols = row.find_all("td")
            table_data[cols[0].text] = cols[1].text

        ag.dividende = float(re.compile("(0\.?\d{0,2})%").findall(table_data.get("Dividende"))[0])
        ag.max_zertis = int(re.compile("(\d{1,2})%").findall(table_data.get("Max. Zertifikatevol."))[0])
        ag.tages_hoch = float(re.compile("\d*\.?\d*,?\d*").findall(table_data.get("Tageshoch"))[0].replace(".", "").
                              replace(",", "."))
        ag.tages_tief = float(re.compile("\d*\.?\d*,?\d*").findall(table_data.get("Tagestief"))[0].replace(".", "").
                              replace(",", "."))

        return ag

    @staticmethod
    def _create_ceo(ceo_data:dict, web_data:BeautifulSoup) -> Ceo:
        """
        Diese Methode erstellt ein Objekt der Klasse CEO und befüllt dieses mit den übergebenen Werten.

        :param ceo_data: Ein Dictionary, welches die Daten aus der API über den CEO enthält
        :param web_data: Ein BeatifulSoup Objekt, welches die Website der AG des CEO enthält
        :return: Ein Objekt der Klasse CEO, welches die gewünschten Werte enthält
        """
        name = ceo_data.get("name")
        try:
            index = re.compile("Spielerindex:.(.*)").findall(web_data.find("img", attrs={"width":"150"}).attrs.get("title"))[0]
        except AttributeError: # Für den Fall das Spieler in keinem Index ist
            index = None
        registrierung_datum = datetime.strptime(ceo_data.get("registrierung_datum"), "%Y-%m-%d %H:%M:%S")
        gesperrt = ceo_data.get("gesperrt")=="true"
        userprojekt = ceo_data.get("ist_userprojekt_account")=="true"
        return Ceo(name=name, index=index, registrierung_datum=registrierung_datum, gesperrt=gesperrt, userprojekt=userprojekt)

    @staticmethod
    def _convert_number(number:str) -> float:
        """
        Diese Methode wandelt Zahlen in der Form "5,6 Mio." in Gleitkommazahlen um.

        :param number: Die umzuwandelnde Zahl
        :return: Die umgewandelte Gleitkommazahl
        """
        ergebnis = float(re.compile("\d+[,]?\d*").findall(number)[0].replace(",", "."))
        multiplier = re.compile("\d*[,]?\d*.(:Mrd\.|Mio\.|Tsd\.)").findall(number)
        if multiplier:
            multiplier = multiplier[0]
            if multiplier == "Mrd.":
                ergebnis *= 1000000000
            elif multiplier == "Mio.":
                ergebnis *= 1000000
            elif multiplier == "Tsd.":
                ergebnis *= 1000
            else:
                raise ValueError()

        return ergebnis


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

class Markt:
    def __init__(self, ags:int=None, orders_24:int=None, volumen_24:float=None, agsx_punkte:int=None,
                 agsx_aenderung:int=None, put_hebel:float=None, call_hebel:float=None, anleihenzins:float=None):
        self.ags:int = ags
        self.orders_24:int = orders_24
        self.volumen_24:float = volumen_24
        self.agsx_punkte:int = agsx_punkte
        self.agsx_aenderung:int = agsx_aenderung
        self.put_hebel:float = put_hebel
        self.call_hebel:float = call_hebel
        self.anleihenzins:float = anleihenzins

    @property
    def ags(self) -> int:
        return self._ags

    @ags.setter
    def ags(self, value:int):
        self._ags = value

    @property
    def orders_24(self) -> int:
        return self._orders_24

    @orders_24.setter
    def orders_24(self, value:int):
        self._orders_24 = value

    @property
    def volumen_24(self) -> float:
        return self._volumen_24

    @volumen_24.setter
    def volumen_24(self, value:float):
        self._volumen_24 = value

    @property
    def agsx_punkte(self) -> int:
        return self._agsx_punkte

    @agsx_punkte.setter
    def agsx_punkte(self, value:int):
        self._agsx_punkte = value

    @property
    def agsx_aenderung(self) -> int:
        return self._agsx_aenderung

    @agsx_aenderung.setter
    def agsx_aenderung(self, value:int):
        self._agsx_aenderung = value

    @property
    def put_hebel(self) -> float:
        return self._put_hebel

    @put_hebel.setter
    def put_hebel(self, value:float):
        self._put_hebel = value

    @property
    def call_hebel(self) -> float:
        return self._call_hebel

    @call_hebel.setter
    def call_hebel(self, value:float):
        self._call_hebel = value

    @property
    def anleihenzins(self) -> float:
        return self._anleihenzins

    @anleihenzins.setter
    def anleihenzins(self, value:float):
        self._anleihenzins = value


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from datetime import datetime

class Ceo:
    def __init__(self, name:str, index:str, registrierung_datum:datetime, gesperrt:bool, userprojekt:bool):
        self.name:str = name
        self.index:str = index
        self.registrierung_datum:datetime = registrierung_datum
        self.gesperrt:bool = gesperrt
        self.userprojekt:bool = userprojekt

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value:str):
        self._name = value

    @property
    def index(self) -> str:
        return self._index

    @index.setter
    def index(self, value: str):
        self._index = value

    @property
    def registrierung_datum(self) -> datetime:
        return self._registrierung_datum

    @registrierung_datum.setter
    def registrierung_datum(self, value:datetime):
        self._registrierung_datum = value

    @property
    def gesperrt(self) -> bool:
        return self._gesperrt

    @gesperrt.setter
    def gesperrt(self, value:bool):
        self._gesperrt = value

    @property
    def userprojekt(self) -> bool:
        return self._userprojekt

    @userprojekt.setter
    def userprojekt(self, value:bool):
        self._userprojekt = value


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

class Aktie:
    def __init__(self, wkn:int, stueckzahl:int):
        self.wkn:int = wkn
        self.stueckzahl:int = stueckzahl

    @property
    def wkn(self) -> int:
        return self._wkn

    @wkn.setter
    def wkn(self, value:int):
        self._wkn = value

    @property
    def stueckzahl(self) -> int:
        return self._stueckzahl

    @stueckzahl.setter
    def stueckzahl(self, value:int):
        self._stueckzahl = value

class Aktionaer(Aktie):
    def __init__(self, wkn: int, stueckzahl: int):
        super().__init__(wkn, stueckzahl)


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from datetime import datetime

class Order:
    def __init__(self, typ:str, limit:float, stueckzahl:int, orderregel:bool, systembank:bool, datum:datetime):
        self.typ:str = typ
        self.limit:float = limit
        self.stueckzahl:int = stueckzahl
        self.orderregel:bool = orderregel
        self.systembank:bool = systembank
        self.datum:datetime = datum

    @property
    def typ(self) -> str:
        return self._typ

    @typ.setter
    def typ(self, value:str):
        self._typ = value

    @property
    def limit(self) -> float:
        return self._limit

    @limit.setter
    def limit(self, value:float):
        self._limit = value

    @property
    def stueckzahl(self) -> int:
        return self._stueckzahl

    @stueckzahl.setter
    def stueckzahl(self, value:int):
        self._stueckzahl = value

    @property
    def orderregel(self) -> bool:
        return self._orderregel

    @orderregel.setter
    def orderregel(self, value:bool):
        self._orderregel = value

    @property
    def systembank(self) -> bool:
        return self._systembank

    @systembank.setter
    def systembank(self, value:bool):
        self._systembank = value

    @property
    def datum(self) -> datetime:
        return self._datum

    @datum.setter
    def datum(self, value:datetime):
        self._datum = value


#  Copyright (c) 2020 | KingKevin23 (@kingkevin023)

from datetime import datetime

class Zertifikat:
    def __init__(self, betrag:float, typ:str, hebel:float, punkte:int, ablaufdatum:datetime):
        self.betrag:float = betrag
        self.typ:str = typ
        self.hebel:float = hebel
        self.punkte:int = punkte
        self.ablaufdatum:datetime = ablaufdatum

    @property
    def betrag(self) -> float:
        return self._betrag

    @betrag.setter
    def betrag(self, value:float):
        self._betrag = value

    @property
    def typ(self) -> str:
        return self._typ

    @typ.setter
    def typ(self, value:str):
        self._typ = value

    @property
    def hebel(self) -> float:
        return self._hebel

    @hebel.setter
    def hebel(self, value:float):
        self._hebel = value

    @property
    def punkte(self) -> int:
        return self._punkte

    @punkte.setter
    def punkte(self, value:int):
        self._punkte = value

    @property
    def ablaufdatum(self) -> datetime:
        return self._ablaufdatum

    @ablaufdatum.setter
    def ablaufdatum(self, value:datetime):
        self._ablaufdatum = value